=== modified file 'drizzled/serialize/Makefile.am'
--- drizzled/serialize/Makefile.am	2008-11-28 11:49:55 +0000
+++ drizzled/serialize/Makefile.am	2008-12-02 01:40:12 +0000
@@ -15,7 +15,7 @@
 
 #called from the top level Makefile
 
-LDADD = libserialize.la ${PROTOBUF_LIBS}
+LDADD = libserialize.la ${PROTOBUF_LIBS} -luuid
 
 AM_CPPFLAGS = ${PROTOBUF_CFLAGS}
 libserialize_la_CXXFLAGS =
@@ -29,6 +29,8 @@
 bin_PROGRAMS = \
 	       table_reader \
 	       table_writer \
+	       replication_event_reader \
+	       replication_event_writer \
                master_list_reader \
                master_list_writer \
                binlog_reader \
@@ -41,11 +43,13 @@
 
 libserialize_la_SOURCES = \
 			  table.pb.cc \
+			  replication_event.pb.cc \
 			  master_list.pb.cc \
 			  binary_log.pb.cc \
 			  schema.pb.cc
 
 noinst_HEADERS = table.pb.h \
+		 replication_event.pb.h \
 		 master_list.pb.h \
 		 schema.pb.h \
 		 binary_log.pb.h \
@@ -75,13 +79,20 @@
 master_list_reader_SOURCES =  master_list_reader.cc
 master_list_reader_CXXFLAGS =
 
+replication_event_writer_SOURCES =  replication_event_writer.cc
+replication_event_writer_CXXFLAGS =
+
+replication_event_reader_SOURCES =  replication_event_reader.cc
+replication_event_reader_CXXFLAGS =
+
 length_SOURCES = length.cc
 length_CXXFLAGS = -fexceptions
 
-EXTRA_DIST = schema.proto table.proto binary_log.proto master_list.proto
+EXTRA_DIST = schema.proto table.proto binary_log.proto master_list.proto replication_event.proto
 BUILT_SOURCES = schema.pb.h \
 		table.pb.h \
 		binary_log.pb.h \
+		replication_event.pb.h \
 		master_list.pb.h
 
 CLEANFILES =  binary_log.pb.h \
@@ -90,5 +101,7 @@
               schema.pb.cc \
               table.pb.h \
               table.pb.cc \
+	      replication_event.pb.h \
+              replication_event.pb.cc \
               master_list.pb.h \
               master_list.pb.cc

=== added file 'drizzled/serialize/replication_event.proto'
--- drizzled/serialize/replication_event.proto	1970-01-01 00:00:00 +0000
+++ drizzled/serialize/replication_event.proto	2008-12-02 01:40:12 +0000
@@ -0,0 +1,31 @@
+package drizzle;
+
+message Event {
+  enum Type {
+    DDL = 0;
+    INSERT = 1;
+    DELETE = 2;
+    UPDATE = 3;
+    COMMIT = 4;
+  }
+
+  message Value {
+    repeated string value = 1;
+  }
+
+  required Type type = 1;
+  required bool autocommit = 2;
+  required string server_id = 3;
+  required uint64 query_id = 4;
+  required string transaction_id = 5;
+  required string schema = 6;
+  optional string table = 7;
+  repeated string field_names = 8;
+  optional string primary_key = 9;
+  repeated Value values = 10;
+  optional string sql = 11;
+}
+
+message EventList {
+  repeated Event event = 1;
+}

=== added file 'drizzled/serialize/replication_event_reader.cc'
--- drizzled/serialize/replication_event_reader.cc	1970-01-01 00:00:00 +0000
+++ drizzled/serialize/replication_event_reader.cc	2008-12-02 01:40:12 +0000
@@ -0,0 +1,152 @@
+#include <iostream>
+#include <fstream>
+#include <string>
+#include "replication_event.pb.h"
+using namespace std;
+
+/* 
+  Example reader application for master.info data.
+*/
+
+void printRecord(const drizzle::EventList *list) 
+{
+  using namespace drizzle;
+  uint32_t x;
+
+  for (x= 0; x < list->event_size(); x++) 
+  {
+    const drizzle::Event event= list->event(x);
+
+    if (x != 0)
+      cout << endl << "##########################################################################################" << endl << endl;
+
+    switch (event.type())
+    {
+    case Event::DDL:
+      cout << "DDL ";
+      break;
+    case Event::INSERT:
+      {
+        uint32_t x;
+
+        cout << "INSERT INTO " << event.table() << " (";
+
+        for (x= 0; x < event.field_names_size() ; x++)
+        {
+          if (x != 0)
+            cout << ", ";
+
+          cout << event.field_names(x);
+        }
+
+        cout << ") VALUES " << endl;
+
+        for (x= 0; x < event.values_size(); x++)
+        {
+          uint32_t y;
+          Event_Value values= event.values(x);
+
+          if (x != 0)
+            cout << ", ";
+
+          cout << "(";
+          for (y= 0; y < values.value_size() ; y++)
+          {
+            if (y != 0)
+              cout << ", ";
+
+            cout << "\"" << values.value(y) << "\"";
+          }
+          cout << ")";
+        }
+
+        cout << ";" << endl;
+        break;
+      }
+    case Event::DELETE:
+      {
+        uint32_t x;
+        Event_Value values= event.values(0);
+
+        cout << "DELETE FROM " << event.table() << " WHERE " << event.primary_key() << " IN (";
+
+        for (x= 0; x < values.value_size() ; x++)
+        {
+          if (x != 0)
+            cout << ", ";
+
+          cout << "\"" << values.value(x) << "\"";
+        }
+
+        cout << ")" << endl;
+        break;
+      }
+    case Event::UPDATE:
+      {
+        uint32_t count;
+
+        for (count= 0; count < event.values_size() ; count++)
+        {
+          uint32_t x;
+          Event_Value values= event.values(count);
+
+          cout << "UPDATE "  << event.table() << " SET ";
+
+          for (x= 1; x < values.value_size() ; x++)
+          {
+            if (x != 1)
+              cout << ", ";
+
+            cout << event.field_names(x - 1) << " = \"" << values.value(x) << "\"";
+          }
+
+          cout << " WHERE " << event.primary_key() << " = " << values.value(0) << endl;
+        }
+
+        break;
+      }
+    case Event::COMMIT:
+      cout << "COMMIT" << endl;
+      break;
+    }
+    cout << endl;
+
+    if (event.has_sql())
+      cout << "Original SQL: " << event.sql() << endl;
+
+    cout << "AUTOCOMMIT: " << event.autocommit() << endl;
+    cout << "Server id: " << event.server_id() << endl;
+    cout << "Query id: " << event.query_id() << endl;
+    cout << "Transaction id: " << event.transaction_id() << endl;
+    cout << "Schema: " << event.schema() << endl;
+    if (event.type() != Event::DDL)
+      cout << "Table Name: " << event.table() << endl;
+  }
+}
+
+int main(int argc, char* argv[]) 
+{
+  GOOGLE_PROTOBUF_VERIFY_VERSION;
+
+  if (argc != 2) 
+  {
+    cerr << "Usage:  " << argv[0] << " replication event log " << endl;
+    return -1;
+  }
+
+  drizzle::EventList list;
+
+  {
+    // Read the existing master.info file
+    fstream input(argv[1], ios::in | ios::binary);
+    if (!list.ParseFromIstream(&input)) 
+    {
+      cerr << "Failed to parse master.info." << endl;
+      return -1;
+    }
+  }
+
+  printRecord(&list);
+
+  return 0;
+}

=== added file 'drizzled/serialize/replication_event_writer.cc'
--- drizzled/serialize/replication_event_writer.cc	1970-01-01 00:00:00 +0000
+++ drizzled/serialize/replication_event_writer.cc	2008-12-02 01:40:12 +0000
@@ -0,0 +1,136 @@
+#include <iostream>
+#include <fstream>
+#include <string>
+#include <uuid/uuid.h>
+
+#include "replication_event.pb.h"
+using namespace std;
+
+static uint64_t query_id= 0;
+char transaction_id[37];
+
+/* 
+  Example script for reader a Drizzle master replication list.
+*/
+
+void write_ddl(drizzle::Event *record, const char *sql)
+{
+  uuid_t uu;
+
+  uuid_generate_time(uu);
+  uuid_unparse(uu, transaction_id);
+
+  using namespace drizzle;
+  record->set_type(Event::DDL);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(query_id++);
+  record->set_transaction_id(transaction_id);
+  record->set_schema("test");
+  record->set_sql(sql);
+}
+
+void write_insert(drizzle::Event *record, const char *trx)
+{
+  using namespace drizzle;
+  Event::Value *value;
+
+  record->set_type(Event::INSERT);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(query_id++);
+  record->set_transaction_id(trx);
+  record->set_schema("test");
+  record->set_table("t1");
+  record->set_sql("INSERT INTO t1 (a) VALUES (1) (2)");
+
+  /* Add Field Names */
+  record->add_field_names("a");
+
+  /* Add values (first row) */
+  value= record->add_values();
+  value->add_value("1");
+
+  /* Add values (second row) */
+  value= record->add_values();
+  value->add_value("2");
+}
+
+void write_delete(drizzle::Event *record, const char *trx)
+{
+  using namespace drizzle;
+  uuid_t uu;
+  Event::Value *value;
+
+  record->set_type(Event::DELETE);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(query_id++);
+  record->set_transaction_id(trx);
+  record->set_schema("test");
+  record->set_table("t1");
+  record->set_sql("DELETE FROM t1 WHERE a IN (1, 2)");
+
+  /* Add Field Names */
+  record->set_primary_key("a");
+
+  /* Add values for IN() */
+  value= record->add_values();
+  value->add_value("1");
+  value->add_value("2");
+}
+
+void write_update(drizzle::Event *record, const char *trx)
+{
+  using namespace drizzle;
+  Event::Value *value;
+
+  record->set_type(Event::UPDATE);
+  record->set_autocommit(true);
+  record->set_server_id("localhost");
+  record->set_query_id(query_id++);
+  record->set_transaction_id(trx);
+  record->set_schema("test");
+  record->set_table("t1");
+  record->set_sql("UPDATE t1 SET a=5 WHERE a = 1 ");
+  record->set_primary_key("a");
+
+  /* Add Field Names */
+  record->add_field_names("a");
+
+  /* Add values (first row) */
+  value= record->add_values();
+  value->add_value("1"); // The first value is always the primary key comparison value
+  value->add_value("5");
+
+  /* Add values (second row) */
+  value= record->add_values();
+  value->add_value("2");
+  value->add_value("6");
+}
+
+int main(int argc, char* argv[]) 
+{
+  GOOGLE_PROTOBUF_VERIFY_VERSION;
+
+  if (argc != 2) {
+    cerr << "Usage:  " << argv[0] << " REPLICATION_EVENT_LOG " << endl;
+    return -1;
+  }
+
+  drizzle::EventList list;
+
+  write_ddl(list.add_event(), "CREATE TABLE A (a int) ENGINE=innodb");
+  write_insert(list.add_event(), transaction_id);
+  write_delete(list.add_event(), transaction_id);
+  write_update(list.add_event(), transaction_id);
+
+  fstream output(argv[1], ios::out | ios::trunc | ios::binary);
+  if (!list.SerializeToOstream(&output)) 
+  {
+    cerr << "Failed to write replication event log." << endl;
+    return -1;
+  }
+
+  return 0;
+}

